@@ -3,7 +3,8 @@ from popsynth.populations import *
 from popsynth.aux_samplers import *
 
 from popsynth.population import Population
-from popsynth.auxiliary_sampler import AuxiliarySampler, DerivedLumAuxSampler
+from popsynth.auxiliary_sampler import AuxiliarySampler, DerivedLumAuxSampler, NonObservedAuxSampler, AuxiliaryParameter
+from popsynth.distribution import DistributionParameter, LuminosityDistribution, SpatialDistribution
 
 
 import popsynth.populations as populations
@@ -21,13 +22,13 @@ __all__ = [
 ]
 
 
-chance = stats.bernoulli.rvs(0.3)
+# chance = stats.bernoulli.rvs(0.3)
 
-if chance:
+# if chance:
 
-    from IPython.display import display, YouTubeVideo
+#     from IPython.display import display, YouTubeVideo
 
-    display(YouTubeVideo("FYJ1dbyDcrI"))
+#     display(YouTubeVideo("FYJ1dbyDcrI"))
 
 from ._version import get_versions
 
