@@ -1,4 +1,5 @@
 import scipy.stats as stats
+import numpy as np
 
 from popsynth.auxiliary_sampler import AuxiliarySampler, AuxiliaryParameter
 
@@ -32,6 +33,9 @@ class TruncatedNormalAuxSampler(AuxiliarySampler):
             l, u, loc=self.mu, scale=self.tau, size=size,
         )
 
+        assert np.alltrue(self._true_values >= self.lower)
+        assert np.alltrue(self._true_values <= self.upper)
+
     def observation_sampler(self, size):
 
         if self._is_observed:
