@@ -2,17 +2,23 @@ import h5py
 import importlib
 import numpy as np
 import matplotlib.pyplot as plt
-import matplotlib as mpl
-import importlib
+import ipyvolume as ipv
+import pythreejs
+import ipywidgets as widgets
+
+import networkx as nx
+
 
-importlib.import_module("mpl_toolkits.mplot3d").Axes3D
 import pandas as pd
 from IPython.display import display, Math, Markdown
 
-from popsynth.utils.spherical_geometry import sample_theta_phi, xyz
+from popsynth.utils.spherical_geometry import xyz
+from popsynth.utils.array_to_cmap import array_to_cmap
 from popsynth.utils.hdf5_utils import (
     recursively_save_dict_contents_to_group,
     recursively_load_dict_contents_from_group,
+    fill_graph_dict,
+    clean_graph_dict,
 )
 
 from betagen import betagen
@@ -48,6 +54,9 @@ class Population(object):
         truth={},
         hard_cut=False,
         distance_probability=1.0,
+        graph=None,
+        theta=None,
+        phi=None,
     ):
         """
         A population containing all the simulated variables
@@ -87,6 +96,9 @@ class Population(object):
         self._known_distance_idx = known_distance_idx
         self._unknown_distance_idx = unknown_distance_idx
 
+        self._theta = theta
+        self._phi = phi
+
         assert len(known_distances) + len(unknown_distance_idx) == sum(
             selection
         ), "the distances are not the correct size"
@@ -127,6 +139,8 @@ class Population(object):
         self._hard_cut = hard_cut
         self._distance_probability = distance_probability
 
+        self._graph = graph
+
         if sum(self._selection) == 0:
 
             self._no_detection = True
@@ -153,6 +167,10 @@ class Population(object):
 
                 assert len(v) == n_model
 
+    @property
+    def graph(self):
+        return self._graph
+
     @property
     def truth(self):
         """
@@ -165,6 +183,15 @@ class Population(object):
 
         return self._truth
 
+
+    @property
+    def theta(self):
+        return self._theta
+
+    @property
+    def phi(self):
+        return self._phi
+    
     @property
     def luminosities(self):
         """
@@ -325,7 +352,7 @@ class Population(object):
         for k, v in self._auxiliary_quantites.items():
 
             output["%s_obs" % k] = v["obs_values"][self._selection]
-            output["%s_sigma" % k] = v["sigma"]
+            
 
         return output
 
@@ -389,6 +416,8 @@ class Population(object):
             f.create_dataset("fluxes", data=self._fluxes, compression="lzf")
             f.create_dataset("flux_obs", data=self._flux_obs, compression="lzf")
             f.create_dataset("selection", data=self._selection, compression="lzf")
+            f.create_dataset("theta", data=self._theta, compression="lzf")
+            f.create_dataset("phi", data=self._phi, compression="lzf")
 
             aux_grp = f.create_group("auxiliary_quantities")
 
@@ -402,7 +431,7 @@ class Population(object):
                     "obs_values", data=v["obs_values"], compression="lzf"
                 )
 
-                q_grp.attrs["sigma"] = v["sigma"]
+
 
             model_grp = f.create_group("model_spaces")
 
@@ -413,6 +442,10 @@ class Population(object):
             # now store the truths
             recursively_save_dict_contents_to_group(f, "truth", self._truth)
 
+            recursively_save_dict_contents_to_group(
+                f, "graph", fill_graph_dict(nx.to_dict_of_dicts(self._graph))
+            )
+
     @classmethod
     def from_file(cls, file_name):
         """
@@ -464,6 +497,8 @@ class Population(object):
 
             luminosities = f["luminosities"][()]
             distances = f["distances"][()]
+            theta = f["theta"][()]
+            phi = f["phi"][()]
 
             # right now this is just for older pops
             try:
@@ -494,11 +529,14 @@ class Population(object):
                 auxiliary_quantities[str(k)] = {
                     "true_values": f["auxiliary_quantities"][k]["true_values"][()],
                     "obs_values": f["auxiliary_quantities"][k]["obs_values"][()],
-                    "sigma": f["auxiliary_quantities"][k].attrs["sigma"],
                 }
 
             truth = recursively_load_dict_contents_from_group(f, "truth")
 
+            graph = nx.from_dict_of_dicts(
+                clean_graph_dict(recursively_load_dict_contents_from_group(f, "graph"))
+            )
+
         return cls(
             luminosities=luminosities,
             distances=distances,
@@ -524,6 +562,9 @@ class Population(object):
             truth=truth,
             distance_probability=distance_probability,
             hard_cut=hard_cut,
+            graph=graph,
+            theta=theta,
+            phi=phi,
         )
 
     def display(self):
@@ -588,7 +629,7 @@ class Population(object):
             color=flux_color,
             edgecolors="none",
             label="True flux",
-            **kwargs
+            **kwargs,
         )
 
         ax.axhline(self._boundary, color="grey", zorder=-5000, ls="--")
@@ -645,7 +686,7 @@ class Population(object):
             color=flux_color,
             edgecolors="none",
             label="Detected flux",
-            **kwargs
+            **kwargs,
         )
 
         ax.axhline(self._boundary, color="grey", zorder=-5000, ls="--")
@@ -666,7 +707,7 @@ class Population(object):
         obs_color=dark,
         arrow_color="k",
         with_arrows=True,
-        **kwargs
+        **kwargs,
     ):
         """FIXME! briefly describe function
 
@@ -717,225 +758,155 @@ class Population(object):
 
         return fig
 
-    def display_obs_fluxes_sphere(
-        self, ax=None, cmap="magma", distance_transform=None, use_log=False, **kwargs
+    def _display_sphere(
+        self,
+        fluxes,
+        distances,
+        theta,
+        phi,
+        cmap="magma",
+        distance_transform=None,
+        use_log=False,
+        fig=None,
+        background_color="white",
+        show=True,
+        **kwargs,
     ):
-        """FIXME! briefly describe function
 
-        :param ax:
-        :param cmap:
-        :param distance_transform:
-        :param use_log:
-        :returns:
-        :rtype:
-
-        """
-
-        if self._no_detection:
+        if len(fluxes) == 0:
             print("There are no detections to display")
-            if ax is not None:
 
-                fig = ax.get_figure()
+            return
 
-                return fig
-            else:
+        if fig is None:
 
-                return
+            fig = ipv.figure()
 
-        if ax is None:
-            fig, ax = plt.subplots(subplot_kw=dict(projection="3d"))
-
-        else:
-
-            fig = ax.get_figure()
-
-        n = sum(self._selection)
-
-        theta, phi = sample_theta_phi(n)
+        ipv.pylab.style.box_off()
+        ipv.pylab.style.axes_off()
+        ipv.pylab.style.set_style_dark()
+        ipv.pylab.style.background_color(background_color)
 
         if distance_transform is not None:
 
-            distance = distance_transform(self._distance_selected)
+            distance = distance_transform(distances)
 
         else:
 
-            distance = self._distance_selected
+            distance = distances
 
-        x, y, z, x2, y2, z2 = _create_sphere_variables(
-            self._r_max, distance, theta, phi
-        )
+        x, y, z = _create_sphere_variables(self._r_max, distance, theta, phi)
 
-        ax.scatter3D(
-            x,
-            y,
-            z,
-            c=self._flux_selected,
-            cmap=cmap,
-            norm=mpl.colors.LogNorm(
-                vmin=min(self._flux_selected), vmax=max(self._flux_selected)
-            ),
-            **kwargs
-        )
+        _, colors = array_to_cmap(fluxes, cmap, use_log=True)
 
-        ax.plot_wireframe(x2, y2, z2, color="grey", alpha=0.9, rcount=4, ccount=2)
+        ipv.scatter(x, y, z, color=colors, marker="sphere", **kwargs)
 
-        ax._axis3don = False
-        ax.set_xlim(-self._r_max, self._r_max)
-        ax.set_ylim(-self._r_max, self._r_max)
-        ax.set_zlim(-self._r_max, self._r_max)
+        r_value = fig
+
+        if show:
+
+            ipv.xyzlim(self._r_max)
+            fig.camera.up = [1, 0, 0]
+            control = pythreejs.OrbitControls(controlling=fig.camera)
+            fig.controls = control
+            control.autoRotate = True
+            fig.render_continuous = True
+            control.autoRotate = True
+            toggle_rotate = widgets.ToggleButton(description="Rotate")
+            widgets.jslink((control, "autoRotate"), (toggle_rotate, "value"))
+            r_value = toggle_rotate
 
         return fig
 
-    def display_fluxes_sphere(
-        self, ax=None, cmap="magma", distance_transform=None, use_log=False, **kwargs
+    def display_obs_fluxes_sphere(
+        self,
+        cmap="magma",
+        distance_transform=None,
+        use_log=False,
+        background_color="white",
+        show=True,
+        **kwargs,
     ):
 
-        if ax is None:
-            fig, ax = plt.subplots(subplot_kw=dict(projection="3d"))
+        theta = self._theta[self._selection]
+        phi = self._phi[self._selection]
 
-        else:
-
-            fig = ax.get_figure()
-
-        n = len(self._fluxes)
-
-        theta, phi = sample_theta_phi(n)
-
-        if distance_transform is not None:
-
-            distance = distance_transform(self._distances)
-
-        else:
-
-            distance = self._distances
-
-        x, y, z, x2, y2, z2 = _create_sphere_variables(
-            self._r_max, distance, theta, phi
-        )
-
-        ax.scatter3D(
-            x,
-            y,
-            z,
-            c=self._flux_obs,
+        fig = self._display_sphere(
+            self._flux_selected,
+            self._distance_selected,
+            theta=theta,
+            phi=phi,
             cmap=cmap,
-            norm=mpl.colors.LogNorm(vmin=min(self._fluxes), vmax=max(self._fluxes)),
-            **kwargs
+            distance_transform=distance_transform,
+            background_color=background_color,
+            use_log=use_log,
+            show=show,
+            **kwargs,
         )
 
-        ax.plot_wireframe(x2, y2, z2, color="grey", alpha=0.9, rcount=4, ccount=2)
-
-        ax._axis3don = False
-        ax.set_xlim(-self._r_max, self._r_max)
-        ax.set_ylim(-self._r_max, self._r_max)
-        ax.set_zlim(-self._r_max, self._r_max)
+        if show:
+            ipv.show()
 
         return fig
 
     def display_hidden_fluxes_sphere(
-        self, ax=None, cmap="magma", distance_transform=None, use_log=False, **kwargs
+        self,
+        cmap="magma",
+        distance_transform=None,
+        use_log=False,
+        background_color="white",
+        show=True,
+        **kwargs,
     ):
-        """FIXME! briefly describe function
 
-        :param ax:
-        :param cmap:
-        :param distance_transform:
-        :param use_log:
-        :returns:
-        :rtype:
+        theta = self._theta[~self._selection]
+        phi = self._phi[~self._selection]
 
-        """
-
-        # do not display if there is nothing hidden
-        if len(self._selection) == sum(self._selection):
-
-            return
-
-        if ax is None:
-            fig, ax = plt.subplots(subplot_kw=dict(projection="3d"))
-
-        else:
-
-            fig = ax.get_figure()
-
-        n = len(self._flux_hidden)
-
-        theta, phi = sample_theta_phi(n)
-
-        if distance_transform is not None:
-
-            distance = distance_transform(self._distance_hidden)
-
-        else:
-
-            distance = self._distance_hidden
-
-        x, y, z, x2, y2, z2 = _create_sphere_variables(
-            self._r_max, distance, theta, phi
-        )
-
-        ax.scatter3D(
-            x,
-            y,
-            z,
-            c=self._flux_hidden,
+        fig = self._display_sphere(
+            self._flux_hidden,
+            self._distance_hidden,
+            theta=theta,
+            phi=phi,
             cmap=cmap,
-            norm=mpl.colors.LogNorm(
-                vmin=min(self._flux_hidden), vmax=max(self._flux_hidden)
-            ),
-            **kwargs
+            distance_transform=distance_transform,
+            background_color=background_color,
+            use_log=use_log,
+            show=show,
+            **kwargs,
         )
 
-        ax.plot_wireframe(x2, y2, z2, color="grey", alpha=0.9, rcount=4, ccount=2)
-
-        ax._axis3don = False
-        # ax.set_xlim(-self._r_max, self._r_max)
-        # ax.set_ylim(-self._r_max, self._r_max)
-        # ax.set_zlim(-self._r_max, self._r_max)
+        if show:
+            ipv.show()
 
         return fig
 
     def display_flux_sphere(
         self,
-        ax=None,
-        seen_cmap="magma",
-        unseen_cmap="Greys",
+        seen_cmap="Reds",
+        unseen_cmap="Blues",
         distance_transform=None,
         use_log=False,
-        **kwargs
+        background_color="white",
+        **kwargs,
     ):
-        """FIXME! briefly describe function
 
-        :param ax:
-        :param seen_cmap:
-        :param unseen_cmap:
-        :param distance_transform:
-        :param use_log:
-        :returns:
-        :rtype:
-
-        """
-
-        if ax is None:
-            fig, ax = plt.subplots(subplot_kw=dict(projection="3d"))
-
-        else:
-            fig = ax.get_figure()
-
-        self.display_obs_fluxes_sphere(
-            ax=ax,
+        fig = self.display_obs_fluxes_sphere(
             cmap=seen_cmap,
             distance_transform=distance_transform,
             use_log=use_log,
-            **kwargs
+            background_color=background_color,
+            show=False,
+            **kwargs,
         )
 
-        self.display_hidden_fluxes_sphere(
-            ax=ax,
+        fig = self.display_hidden_fluxes_sphere(
             cmap=unseen_cmap,
             distance_transform=distance_transform,
             use_log=use_log,
-            **kwargs
+            background_color=background_color,
+            fig=fig,
+            show=True,
+            **kwargs,
         )
 
         return fig
@@ -1012,10 +983,10 @@ class Population(object):
 def _create_sphere_variables(R, distance, theta, phi):
     x, y, z = xyz(distance, theta, phi)
 
-    u = np.linspace(0, 2 * np.pi, 100)
-    v = np.linspace(0, np.pi, 100)
-    x2 = R * np.outer(np.cos(u), np.sin(v))
-    y2 = R * np.outer(np.sin(u), np.sin(v))
-    z2 = R * np.outer(np.ones(np.size(u)), np.cos(v))
+    # u = np.linspace(0, 2 * np.pi, 100)
+    # v = np.linspace(0, np.pi, 100)
+    # x2 = R * np.outer(np.cos(u), np.sin(v))
+    # y2 = R * np.outer(np.sin(u), np.sin(v))
+    # z2 = R * np.outer(np.ones(np.size(u)), np.cos(v))
 
-    return x, y, z, x2, y2, z2
+    return x, y, z
